# ----------------------------------------------------------------------
# Copyright (C) 2015 by Rafael Gonzalez 
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
# 
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
# ----------------------------------------------------------------------

from twisted.trial import unittest
from twisted.test import proto_helpers

from mqtt import v31, v311
from mqtt.pdu import (
    CONNECT,
    CONNACK,
    DISCONNECT,
    PINGREQ,
    PINGRES,
    SUBSCRIBE,
    SUBACK,
    UNSUBSCRIBE,
    UNSUBACK,
    PUBLISH,
    PUBACK,
    PUBREC,
    PUBREL,
    PUBCOMP,
    )

class PDUTestCase(unittest.TestCase):
        

    def test_CONNECT_encdec(self):
        request = CONNECT()
        response = CONNECT()
        request.clientId    = "client-foo"
        request.version     = v31
        request.keepalive   = 0
        request.willTopic   = None
        request.willMessage = None
        request.willQoS     = None
        request.willRetain  = None
        request.username    = None
        request.password    = None
        request.cleanStart  = True
        response.decode(request.encode())
        self.assertEqual(request.encoded[0],  response.encoded[0])
        self.assertEqual(request.version,     response.version)
        self.assertEqual(request.clientId,    response.clientId)
        self.assertEqual(request.keepalive,   response.keepalive)
        self.assertEqual(request.willTopic,   response.willTopic)
        self.assertEqual(request.willMessage, response.willMessage)
        self.assertEqual(request.willQoS,     response.willQoS)
        self.assertEqual(request.willRetain,  response.willRetain)
        self.assertEqual(request.username,    response.username)
        self.assertEqual(request.password,    response.password)
        self.assertEqual(request.cleanStart,  response.cleanStart)
        

        
    def test_CONNECT_encdec_keepalive(self):
        request = CONNECT()
        response = CONNECT()
        request.version     = v31
        request.clientId    = "client-foo"
        request.keepalive   = 12
        request.willTopic   = None
        request.willMessage = None
        request.willQoS     = None
        request.willRetain  = None
        request.username    = None
        request.password    = None
        request.cleanStart  = True
        response.decode(request.encode())
        self.assertEqual(request.encoded[0],  response.encoded[0])
        self.assertEqual(request.version,     response.version)
        self.assertEqual(request.clientId,    response.clientId)
        self.assertEqual(request.keepalive,   response.keepalive)
        self.assertEqual(request.willTopic,   response.willTopic)
        self.assertEqual(request.willMessage, response.willMessage)
        self.assertEqual(request.willQoS,     response.willQoS)
        self.assertEqual(request.willRetain,  response.willRetain)
        self.assertEqual(request.username,    response.username)
        self.assertEqual(request.password,    response.password)
        self.assertEqual(request.cleanStart,  response.cleanStart)
        self.assertEqual(request.version,     response.version)

    def test_CONNECT_encdec_willTopic(self):
        request = CONNECT()
        response = CONNECT()
        request.clientId    = "client-foo"
        request.keepalive   = 1
        request.willTopic   = "foo-topic"
        request.willMessage = ""
        request.willQoS     = 1
        request.willRetain  = False
        request.username    = None
        request.password    = None
        request.cleanStart  = True
        request.version     = v31
        response.decode(request.encode())
        self.assertEqual(request.encoded[0],  response.encoded[0])
        self.assertEqual(request.version,     response.version)
        self.assertEqual(request.clientId,    response.clientId)
        self.assertEqual(request.keepalive,   response.keepalive)
        self.assertEqual(request.willTopic,   response.willTopic)
        self.assertEqual(request.willMessage, response.willMessage)
        self.assertEqual(request.willQoS,     response.willQoS)
        self.assertEqual(request.willRetain,  response.willRetain)
        self.assertEqual(request.username,    response.username)
        self.assertEqual(request.password,    response.password)
        self.assertEqual(request.cleanStart,  response.cleanStart)

    def test_CONNECT_encdec_willMessage(self):
        request = CONNECT()
        response = CONNECT()
        request.clientId    = "client-foo"
        request.keepalive   = 1
        request.willTopic   = "foo-topic"
        request.willMessage = "Hello World"
        request.willQoS     = 2
        request.willRetain  = False
        request.username    = None
        request.password    = None
        request.cleanStart  = True
        request.version     = v31
        response.decode(request.encode())
        self.assertEqual(request.encoded[0],  response.encoded[0])
        self.assertEqual(request.version,     response.version)
        self.assertEqual(request.clientId,    response.clientId)
        self.assertEqual(request.keepalive,   response.keepalive)
        self.assertEqual(request.willTopic,   response.willTopic)
        self.assertEqual(request.willMessage, response.willMessage)
        self.assertEqual(request.willQoS,     response.willQoS)
        self.assertEqual(request.willRetain,  response.willRetain)
        self.assertEqual(request.username,    response.username)
        self.assertEqual(request.password,    response.password)
        self.assertEqual(request.cleanStart,  response.cleanStart)

    def test_CONNECT_encdec_willRetain(self):
        request = CONNECT()
        response = CONNECT()
        request.clientId    = "client-foo"
        request.keepalive   = 1
        request.willTopic   = "foo-topic"
        request.willMessage = "Hello World"
        request.willQoS     = 2
        request.willRetain  = True
        request.username    = None
        request.password    = None
        request.cleanStart  = True
        request.version     = v31
        response.decode(request.encode())
        self.assertEqual(request.encoded[0],  response.encoded[0])
        self.assertEqual(request.version,     response.version)
        self.assertEqual(request.clientId,    response.clientId)
        self.assertEqual(request.keepalive,   response.keepalive)
        self.assertEqual(request.willTopic,   response.willTopic)
        self.assertEqual(request.willMessage, response.willMessage)
        self.assertEqual(request.willQoS,     response.willQoS)
        self.assertEqual(request.willRetain,  response.willRetain)
        self.assertEqual(request.username,    response.username)
        self.assertEqual(request.password,    response.password)
        self.assertEqual(request.cleanStart,  response.cleanStart)
        

    def test_CONNECT_encdec_userpass(self):
        request = CONNECT()
        response = CONNECT()
        request.clientId    = "client-foo"
        request.keepalive   = 12000
        request.willTopic   = "foo-topic"
        request.willMessage = ""
        request.willQoS     = 0
        request.willRetain  = False
        request.username    = "foouser"
        request.password    = "foopasswd"
        request.cleanStart  = True
        request.version     = v31
        response.decode(request.encode())
        self.assertEqual(request.encoded[0],  response.encoded[0])
        self.assertEqual(request.version,     response.version)
        self.assertEqual(request.clientId,    response.clientId)
        self.assertEqual(request.keepalive,   response.keepalive)
        self.assertEqual(request.willTopic,   response.willTopic)
        self.assertEqual(request.willMessage, response.willMessage)
        self.assertEqual(request.willQoS,     response.willQoS)
        self.assertEqual(request.willRetain,  response.willRetain)
        self.assertEqual(request.username,    response.username)
        self.assertEqual(request.password,    response.password.decode(encoding='ascii', errors='ignore'))
        self.assertEqual(request.cleanStart,  response.cleanStart)
       
    def test_CONNECT_encdec_session(self):
        request = CONNECT()
        response = CONNECT()
        request.clientId    = "client-foo"
        request.keepalive   = 1200
        request.willTopic   = "foo-topic"
        request.willMessage = ""
        request.willQoS     = 1
        request.willRetain  = False
        request.username    = None
        request.password    = None
        request.cleanStart  = False
        request.version     = v31
        response.decode(request.encode())
        self.assertEqual(request.encoded[0],  response.encoded[0])
        self.assertEqual(request.version,     response.version)
        self.assertEqual(request.clientId,    response.clientId)
        self.assertEqual(request.keepalive,   response.keepalive)
        self.assertEqual(request.willTopic,   response.willTopic)
        self.assertEqual(request.willMessage, response.willMessage)
        self.assertEqual(request.willQoS,     response.willQoS)
        self.assertEqual(request.willRetain,  response.willRetain)
        self.assertEqual(request.username,    response.username)
        self.assertEqual(request.password,    response.password)
        self.assertEqual(request.cleanStart,  response.cleanStart)
        

    def test_CONNECT_encdec_version(self):
        request = CONNECT()
        response = CONNECT()
        request.clientId    = "client-foo"
        request.keepalive   = 120
        request.willTopic   = "foo-topic"
        request.willMessage = ""
        request.willQoS     = 0
        request.willRetain  = False
        request.username    = None
        request.password    = None
        request.cleanStart  = True
        request.version     = v311  
        response.decode(request.encode())
        self.assertEqual(request.encoded[0],  response.encoded[0])
        self.assertEqual(request.version,     response.version)
        self.assertEqual(request.clientId,    response.clientId)
        self.assertEqual(request.keepalive,   response.keepalive)
        self.assertEqual(request.willTopic,   response.willTopic)
        self.assertEqual(request.willMessage, response.willMessage)
        self.assertEqual(request.willQoS,     response.willQoS)
        self.assertEqual(request.willRetain,  response.willRetain)
        self.assertEqual(request.username,    response.username)
        self.assertEqual(request.password,    response.password)
        self.assertEqual(request.cleanStart,  response.cleanStart)
       


    def test_PINGREQ_encdec(self):
        request = PINGREQ()
        response = PINGREQ()
        
        response.decode(request.encode())
        self.assertEqual(request.encoded[0],  response.encoded[0])

    def test_PINGRES_encdec(self):
        request = PINGRES()
        response = PINGRES()
        
        response.decode(request.encode())
        self.assertEqual(request.encoded[0], response.encoded[0])

    def test_DISCONNECT_encdec(self):
        request = DISCONNECT()
        response = DISCONNECT()
        
        response.decode(request.encode())
        self.assertEqual(request.encoded[0],  response.encoded[0])

    def test_CONNACK_encdec(self):
        request = CONNACK()
        response = CONNACK()
        request.session    = True
        request.resultCode = 2
        
        response.decode(request.encode())
        self.assertEqual(request.encoded[0],  response.encoded[0])
        self.assertEqual(request.session,     response.session)
        self.assertEqual(request.resultCode,  response.resultCode)

    def test_SUBSCRIBE_encdec(self):
        request = SUBSCRIBE()
        response = SUBSCRIBE()
        request.topics = [('foo', 1), ('bar',0), ('baz',2)]
        request.msgId  = 5
        
        response.decode(request.encode())
        self.assertEqual(request.msgId,  response.msgId)
        self.assertEqual(request.topics, response.topics)
       
    def test_SUBACK_encdec(self):
        request  = SUBACK()
        response = SUBACK()
        request.msgId = 5
        request.granted = [(0, False), (0, True), (1,False), (1,True), (2,False), (2,True)]
        
        response.decode(request.encode())
        self.assertEqual(request.msgId,  response.msgId)
        self.assertEqual(request.granted, response.granted)

    def test_UNSUBSCRIBE_encdec(self):
        request  = UNSUBSCRIBE()
        response = UNSUBSCRIBE()
        request.topics = ['foo', 'bar', 'baz']
        request.msgId = 6
        
        response.decode(request.encode())
        self.assertEqual(request.msgId,  response.msgId)
        self.assertEqual(request.topics, response.topics)

    def test_UNSUBACK_encdec(self):
        request  = UNSUBACK()
        response = UNSUBACK()
        request.msgId = 5
        
        response.decode(request.encode())
        self.assertEqual(request.msgId, response.msgId)

    def test_PUBACK_encdec(self):
        request  = PUBACK()
        response = PUBACK()
        request.msgId = 65535
        
        response.decode(request.encode())
        self.assertEqual(request.msgId, response.msgId)

    def test_PUBREC_encdec(self):
        request  = PUBREC()
        response = PUBREC()
        request.msgId = 30001
        
        response.decode(request.encode())
        self.assertEqual(request.msgId, response.msgId)

    def test_PUBREL_encdec(self):
        request  = PUBREL()
        response = PUBREL()
        request.msgId = 30002
        
        response.decode(request.encode())
        self.assertEqual(request.msgId, response.msgId)

    def test_PUBCOMP_encdec(self):
        request  = PUBCOMP()
        response = PUBCOMP()
        request.msgId = 30002
        
        response.decode(request.encode())
        self.assertEqual(request.msgId, response.msgId)

    def test_PUBLISH_encdec(self):
        request  = PUBLISH()
        response = PUBLISH()
        request.msgId   = None
        request.qos     = 0
        request.dup     = False
        request.retain  = False
        request.topic   = "foo"
        request.payload = "foo"
        
        response.decode(request.encode())
        self.assertEqual(request.msgId,   response.msgId)
        self.assertEqual(request.qos,     response.qos)
        self.assertEqual(request.dup,     response.dup)
        self.assertEqual(request.retain,  response.retain)
        self.assertEqual(request.topic,   response.topic)
        self.assertEqual(request.payload, response.payload.decode(encoding='utf-8'))

    def test_PUBLISH_encdec_qos(self):
        request  = PUBLISH()
        response = PUBLISH()
        request.msgId   = 30001
        request.qos     = 1
        request.dup     = False
        request.retain  = False
        request.topic   = "foo"
        request.payload = "foo"
        
        response.decode(request.encode())
        self.assertEqual(request.msgId,   response.msgId)
        self.assertEqual(request.qos,     response.qos)
        self.assertEqual(request.dup,     response.dup)
        self.assertEqual(request.retain,  response.retain)
        self.assertEqual(request.topic,   response.topic)
        self.assertEqual(request.payload, response.payload.decode(encoding='utf-8'))

    def test_PUBLISH_encdec_dup(self):
        request  = PUBLISH()
        response = PUBLISH()
        request.msgId   = 30001
        request.qos     = 1
        request.dup     = True
        request.retain  = False
        request.topic   = "foo"
        request.payload = "foo"
        
        response.decode(request.encode())
        self.assertEqual(request.msgId,   response.msgId)
        self.assertEqual(request.qos,     response.qos)
        self.assertEqual(request.dup,     response.dup)
        self.assertEqual(request.retain,  response.retain)
        self.assertEqual(request.topic,   response.topic)
        self.assertEqual(request.payload, response.payload.decode(encoding='utf-8'))

    def test_PUBLISH_encdec_retain(self):
        request  = PUBLISH()
        response = PUBLISH()
        request.msgId   = 30001
        request.qos     = 1
        request.dup     = False
        request.retain  = True
        request.topic   = "foo"
        request.payload = "foo"
        
        response.decode(request.encode())
        self.assertEqual(request.msgId,   response.msgId)
        self.assertEqual(request.qos,     response.qos)
        self.assertEqual(request.dup,     response.dup)
        self.assertEqual(request.retain,  response.retain)
        self.assertEqual(request.topic,   response.topic)
        self.assertEqual(request.payload, response.payload.decode(encoding='utf-8'))

    def test_PUBLISH_encdec_payload_str(self):
        request  = PUBLISH()
        response = PUBLISH()
        request.msgId   = 30001
        request.qos     = 1
        request.dup     = False
        request.retain  = True
        request.topic   = "foo"
        request.payload = ""
        
        response.decode(request.encode())
        self.assertEqual(request.msgId,   response.msgId)
        self.assertEqual(request.qos,     response.qos)
        self.assertEqual(request.dup,     response.dup)
        self.assertEqual(request.retain,  response.retain)
        self.assertEqual(request.topic,   response.topic)
        self.assertEqual(request.payload, response.payload.decode(encoding='utf-8'))

       
    def test_PUBLISH_encdec_payload_bytearray(self):
        request  = PUBLISH()
        response = PUBLISH()
        request.msgId   = 30001
        request.qos     = 1
        request.dup     = False
        request.retain  = True
        request.topic   = "foo"
        request.payload = bytearray(5)
        
        response.decode(request.encode())
        self.assertEqual(request.msgId,   response.msgId)
        self.assertEqual(request.qos,     response.qos)
        self.assertEqual(request.dup,     response.dup)
        self.assertEqual(request.retain,  response.retain)
        self.assertEqual(request.topic,   response.topic)
        self.assertEqual(request.payload, response.payload)


class PDUTestCase2(unittest.TestCase):

    def test_PUBREC_enc_fail1(self):
        request  = PUBACK()
        response = PUBACK()
        request.msgId = -1
        self.assertRaises(ValueError, request.encode)


    def test_PUBREC_enc_fail2(self):
        request  = PUBACK()
        response = PUBACK()
        request.msgId = 2000000
        self.assertRaises(ValueError, request.encode)

    def test_PUBLISH_encdec_payload_int(self):
        request  = PUBLISH()
        request.msgId   = 30001
        request.qos     = 1
        request.dup     = False
        request.retain  = True
        request.topic   = "foo"
        request.payload = 65537
        self.assertRaises(TypeError, request.encode)

    def test_PUBLISH_encdec_payload_float(self):
        request  = PUBLISH()
        response = PUBLISH()
        request.msgId   = 30001
        request.qos     = 1
        request.dup     = False
        request.retain  = True
        request.topic   = "foo"
        request.payload = 12.25
        self.assertRaises(TypeError, request.encode)
    
        



# ----------------------------------------------------------------------
# Copyright (C) 2015 by Rafael Gonzalez 
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
# 
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
# ----------------------------------------------------------------------

from twisted.trial    import unittest
from twisted.test     import proto_helpers
from twisted.internet import task, defer, error

from twisted.internet.address import IPv4Address


from mqtt                   import v31
from mqtt.error             import MQTTWindowError
from mqtt.pdu               import CONNACK, SUBSCRIBE, SUBACK, UNSUBACK, PUBLISH, PUBREL
from mqtt.client.factory    import MQTTFactory
from mqtt.client.base       import MQTTBaseProtocol
from mqtt.client.subscriber import MQTTProtocol as MQTTSubscriberProtocol
from mqtt.client.publisher  import MQTTProtocol as MQTTPublisherProtocol
from mqtt.client.pubsubs    import MQTTProtocol as MQTTPubSubsProtocol




class TestMQTTSubscriber1(unittest.TestCase):


    def setUp(self):
        '''
        Set up a conencted state
        '''
        self.transport = proto_helpers.StringTransportWithDisconnection()
        self.clock     = task.Clock()
        MQTTBaseProtocol.callLater = self.clock.callLater
        self.factory   = MQTTFactory(MQTTFactory.SUBSCRIBER)
        self.addr = IPv4Address('TCP','localhost',1880)
        self._rebuild()
        self._connect()

    def _connect(self, cleanStart=True):
        '''
        Go to connected state
        '''
        ack = CONNACK()
        ack.session = False
        ack.resultCode = 0
        ack.encode()
        self.protocol.connect("TwistedMQTT-sub", keepalive=0, cleanStart=cleanStart, version=v31)
        self.transport.clear()
        self.protocol.dataReceived(ack.encoded)


    def _serverDown(self):
        self.transport.loseConnection()
        self.transport.clear()
        del self.protocol

    def _rebuild(self):
        self.protocol  = self.factory.buildProtocol(self.addr)
        self.transport.protocol = self.protocol
        MQTTBaseProtocol.callLater = self.clock.callLater
        self.protocol.makeConnection(self.transport)


    def _subscribe(self, n, qos, topic):
        self.protocol.setWindowSize(n)
        dl = []
        for i in range(0,n):
            t = "{0}{1}".format(topic, i)
            dl.append(self.protocol.subscribe(t, qos))
        self.transport.clear()
        for d in dl:
            self.assertNoResult(d)
        return dl

    def _unsubscribe(self, n, topic):
        self.protocol.setWindowSize(n)
        dl = []
        for i in range(0,n):
            t = "{0}{1}".format(topic, i)
            dl.append(self.protocol.unsubscribe(t))
        self.transport.clear()
        for d in dl:
            self.assertNoResult(d)
        return dl

    def test_subscribe_single(self):
        d = self.protocol.subscribe("foo/bar/baz1", 2 )
        self.transport.clear()
        ack = SUBACK()
        ack.msgId = d.msgId
        ack.granted = [(2, False)]
        self.protocol.dataReceived(ack.encode())
        self.assertEqual([(2, False)], self.successResultOf(d))

    def test_subscribe_single_large_qos(self):
        d = self.protocol.subscribe("foo/bar/baz1", 3)
        self.transport.clear()
        self.failureResultOf(d).trap(ValueError)

    def test_subscribe_single_negative_qos(self):
        d = self.protocol.subscribe("foo/bar/baz1", -1)
        self.transport.clear()
        self.failureResultOf(d).trap(ValueError)

    def test_subscribe_tuple(self):
        d = self.protocol.subscribe( ("foo/bar/baz1", 2) )
        self.transport.clear()
        ack = SUBACK()
        ack.msgId = d.msgId
        ack.granted = [(2, False)]
        self.protocol.dataReceived(ack.encode())
        self.assertEqual([(2, False)], self.successResultOf(d))

    def test_subscribe_list(self):
        d = self.protocol.subscribe( [ ("foo/bar/baz1", 2), ("foo/bar/baz2", 1), ("foo/bar/baz3", 0) ] )
        d.addCallback(self.assertEqual, [(2, False), (1, False), (0, False)] )
        self.transport.clear()
        ack = SUBACK()
        ack.msgId = d.msgId
        ack.granted = [(2, False), (1, False), (0, False)]
        self.protocol.dataReceived(ack.encode())
        self.assertEqual( [(2, False), (1, False), (0, False)], self.successResultOf(d))

    def test_subscribe_several_fail(self):
        dl = self._subscribe(n=3, qos=2, topic="foo/bar/baz")
        self.assertEqual(len(self.protocol.factory.windowSubscribe[self.addr]), 3)
        self._serverDown()
        for d in dl:
            self.failureResultOf(d).trap(error.ConnectionDone)
        

    def test_subscribe_several_window_fail(self):
        self.protocol.setWindowSize(3)
        dl = self._subscribe(n=3, qos=2, topic="foo/bar/baz")
        self.assertEqual(len(self.protocol.factory.windowSubscribe[self.addr]), 3)
        d4 = self.protocol.subscribe("foo/bar/baz3", 2 )
        self.assertEqual(len(self.protocol.factory.windowSubscribe[self.addr]), 3)
        self.failureResultOf(d4).trap(MQTTWindowError)
        self._serverDown()
        for d in dl:
            self.failureResultOf(d).trap(error.ConnectionDone)
        

    def test_unsubscribe_single(self):
        d = self.protocol.unsubscribe("foo/bar/baz1")
        self.transport.clear()
        ack = UNSUBACK()
        ack.msgId = d.msgId
        self.protocol.dataReceived(ack.encode())
        self.assertEqual(ack.msgId, self.successResultOf(d))


    def test_unsubscribe_list(self):
        d = self.protocol.unsubscribe( [ "foo/bar/baz1", "foo/bar/baz2", "foo/bar/baz3"] )
        self.transport.clear()
        ack = UNSUBACK()
        ack.msgId = d.msgId
        self.protocol.dataReceived(ack.encode())
        self.assertEqual(ack.msgId, self.successResultOf(d))

    def test_unsubscribe_several_fail(self):
        dl = self._unsubscribe(n=3, topic="foo/bar/baz")
        self.assertEqual(len(self.protocol.factory.windowUnsubscribe[self.addr]), 3)
        self._serverDown()
        for d in dl:
            self.failureResultOf(d).trap(error.ConnectionDone)

    def test_unsubscribe_several_window_fail(self):
        dl = self._unsubscribe(n=3, topic="foo/bar/baz")
        self.assertEqual(len(self.protocol.factory.windowUnsubscribe[self.addr]), 3)
        d4 = self.protocol.unsubscribe("foo/bar/baz4")
        self.assertEqual(len(self.protocol.factory.windowUnsubscribe[self.addr]), 3)
        self.failureResultOf(d4).trap(MQTTWindowError)
        self._serverDown()
        for d in dl:
            self.failureResultOf(d).trap(error.ConnectionDone)

    def test_publish_recv_qos0(self):
        def onPublish(topic, payload, qos, dup, retain, msgId):
            self.topic   = topic
            self.payload = payload.decode('utf-8')
            self.qos     = qos
            self.retain  = retain
            self.msgId   = msgId 
        self.protocol.onPublish = onPublish
        pub =PUBLISH()
        pub.qos     = 0
        pub.dup     = False
        pub.retain  = False
        pub.topic   = "foo/bar/baz0"
        pub.msgId   = None
        pub.payload = "Hello world 0"
        self.protocol.dataReceived(pub.encode())
        self.assertEqual(self.topic,   pub.topic)
        self.assertEqual(self.payload, pub.payload)
        self.assertEqual(self.qos,     pub.qos)
        self.assertEqual(self.retain,  pub.retain)
        self.assertEqual(self.msgId,   pub.msgId )

    def test_publish_recv_qos1(self):
        def onPublish(topic, payload, qos, dup, retain, msgId):
            self.topic   = topic
            self.payload = payload.decode('utf-8')
            self.qos     = qos
            self.retain  = retain
            self.msgId   = msgId 
            self.dup     = dup
        self.protocol.onPublish = onPublish
        pub =PUBLISH()
        pub.qos     = 1
        pub.dup     = False
        pub.retain  = False
        pub.topic   = "foo/bar/baz1"
        pub.msgId   = 1
        pub.payload = "Hello world 1"
        self.protocol.dataReceived(pub.encode())
        self.transport.clear()
        self.assertEqual(self.topic,   pub.topic)
        self.assertEqual(self.payload, pub.payload)
        self.assertEqual(self.qos,     pub.qos)
        self.assertEqual(self.retain,  pub.retain)
        self.assertEqual(self.msgId,   pub.msgId )
        self.assertEqual(self.dup,     pub.dup )

    def test_publish_recv_qos2(self):
        self.called = False
        def onPublish(topic, payload, qos, dup, retain, msgId):
            self.called = True
            self.topic   = topic
            self.payload = payload.decode('utf-8')
            self.qos     = qos
            self.retain  = retain
            self.msgId   = msgId 
            self.dup     = dup
        self.protocol.onPublish = onPublish
        pub =PUBLISH()
        pub.qos     = 2
        pub.dup     = False
        pub.retain  = False
        pub.topic   = "foo/bar/baz2"
        pub.msgId   = 1
        pub.payload = "Hello world 2"
        self.protocol.dataReceived(pub.encode())
        self.transport.clear()
        self.assertEqual(self.called, False)
        rel = PUBREL()
        rel.msgId = pub.msgId
        self.protocol.dataReceived(rel.encode())
        self.assertEqual(self.topic,   pub.topic)
        self.assertEqual(self.payload, pub.payload)
        self.assertEqual(self.qos,     pub.qos)
        self.assertEqual(self.retain,  pub.retain)
        self.assertEqual(self.msgId,   pub.msgId )
        self.assertEqual(self.dup,     pub.dup )


class TestMQTTSubscriberDisconnect(unittest.TestCase):
    '''
    Testing various cases of disconnect callback
    '''

    def setUp(self):
        '''
        Set up a connencted state
        '''
        self.transport = proto_helpers.StringTransportWithDisconnection()
        self.clock     = task.Clock()
        MQTTBaseProtocol.callLater = self.clock.callLater
        self.factory   = MQTTFactory(MQTTFactory.SUBSCRIBER)
        self._rebuild()
        self.disconnected = False

    def _connect(self, cleanStart=True):
        '''
        Go to connected state
        '''
        ack = CONNACK()
        ack.session = False
        ack.resultCode = 0
        ack.encode()
        self.protocol.connect("TwistedMQTT-sub", keepalive=0, cleanStart=cleanStart, version=v31)
        self.transport.clear()
        self.protocol.dataReceived(ack.encoded)

    def _disconnected(self, reason):
        self.disconnected = True

    def _rebuild(self):
        self.protocol  = self.factory.buildProtocol(0)
        self.transport.protocol = self.protocol
        MQTTBaseProtocol.callLater = self.clock.callLater
        self.protocol.makeConnection(self.transport)

    def _serverDown(self):
        self.transport.loseConnection()
        self.transport.clear()
        del self.protocol

    def test_disconnect_1(self):
        '''Just connect and lose the transport'''
        self._connect()
        self.protocol.onDisconnection = self._disconnected
        self.transport.loseConnection()
        self.assertEqual(self.disconnected, True)

    def test_disconnect_2(self):
        '''connect and disconnect'''
        self._connect()
        self.protocol.onDisconnection = self._disconnected
        self.protocol.disconnect()
        self.assertEqual(self.disconnected, True)

    def test_disconnect_3(self):
        '''connect, generate a deferred and lose the transport'''
        self._connect()
        self.protocol.onDisconnection = self._disconnected
        d = self.protocol.subscribe("foo/bar/baz1", 2 )
        self.transport.clear()
        self.transport.loseConnection()
        self.assertEqual(self.disconnected, True)
        self.failureResultOf(d).trap(error.ConnectionDone)

    def test_disconnect_4(self):
        '''connect, generate a deferred and disconnect'''
        self._connect()
        self.protocol.onDisconnection = self._disconnected
        d = self.protocol.subscribe("foo/bar/baz1", 2 )
        self.transport.clear()
        self.protocol.disconnect()
        self.assertEqual(self.disconnected, True)
        self.failureResultOf(d).trap(error.ConnectionDone)

    def test_disconnect_5(self):
        '''connect with persistent session, 
        enerate a deferred that will not errback 
        and then disconnect'''
        self._connect(cleanStart=False)
        self.protocol.onDisconnection = self._disconnected
        d = self.protocol.subscribe("foo/bar/baz1", 2 )
        self.transport.clear()
        self.protocol.disconnect()
        self.assertEqual(self.disconnected, True)
        self.assertNoResult(d)

    def test_disconnect_6(self):
        '''connect with persistent session, 
        generate a deferred that will not errback yet, 
        then rebuilds protocol'''
        self._connect(cleanStart=False)
        self.protocol.onDisconnection = self._disconnected
        d = self.protocol.subscribe("foo/bar/baz1", 2 )
        self._serverDown()
        self._rebuild()
        self.assertEqual(self.disconnected, True)
        self.assertNoResult(d)


class TestMQTTSubscriber2(unittest.TestCase):


    def setUp(self):
        '''
        Set up a conencted state
        '''
        self.transport = proto_helpers.StringTransportWithDisconnection()
        self.clock     = task.Clock()
        MQTTBaseProtocol.callLater = self.clock.callLater
        self.factory   = MQTTFactory(MQTTFactory.SUBSCRIBER)
        self._rebuild()
       
    def _connect(self, cleanStart=True):
        '''
        Go to connected state
        '''
        ack = CONNACK()
        ack.session = False
        ack.resultCode = 0
        ack.encode()
        self.protocol.connect("TwistedMQTT-sub", keepalive=0, cleanStart=cleanStart, version=v31)
        self.transport.clear()
        self.protocol.dataReceived(ack.encoded)


    def _serverDown(self):
        self.transport.loseConnection()
        self.transport.clear()
        del self.protocol

    def _rebuild(self):
        self.protocol  = self.factory.buildProtocol(0)
        self.transport.protocol = self.protocol
        MQTTBaseProtocol.callLater = self.clock.callLater
        self.protocol.makeConnection(self.transport)


    def _subscribe(self, n, qos, topic):
        self.protocol.setWindowSize(n)
        dl = []
        for i in range(0,n):
            t = "{0}{1}".format(topic, i)
            dl.append(self.protocol.subscribe(t, qos))
        self.transport.clear()
        for d in dl:
            self.assertNoResult(d)
        return dl

    def _unsubscribe(self, n, topic):
        self.protocol.setWindowSize(n)
        dl = []
        for i in range(0,n):
            t = "{0}{1}".format(topic, i)
            dl.append(self.protocol.unsubscribe(t))
        self.transport.clear()
        for d in dl:
            self.assertNoResult(d)
        return dl

    def test_subscribe_setPubishHandler1(self):
        def onPublish(topic, payload, qos, dup, retain, msgId):
            self.called  = True
        self.protocol.onPublish = onPublish
        self._connect()
        d = self.protocol.subscribe("foo/bar/baz1", 2 )
        self.transport.clear()
        ack = SUBACK()
        ack.msgId = d.msgId
        ack.granted = [(2, False)]
        self.protocol.dataReceived(ack.encode())
        self.assertEqual([(2, False)], self.successResultOf(d))

    def test_subscribe_setPubishHandler2(self):
        def onPublish(topic, payload, qos, dup, retain, msgId):
            self.called  = True
        self._connect()
        self.protocol.onPublish = onPublish
        d = self.protocol.subscribe("foo/bar/baz1", 2 )
        self.transport.clear()
        ack = SUBACK()
        ack.msgId = d.msgId
        ack.granted = [(2, False)]
        self.protocol.dataReceived(ack.encode())
        self.assertEqual([(2, False)], self.successResultOf(d))


import sys

from twisted.internet.defer       import inlineCallbacks, DeferredList
from twisted.internet             import reactor, task
from twisted.internet.endpoints   import clientFromString
from twisted.application.internet import ClientService, backoffPolicy

from twisted.logger   import (
    Logger, LogLevel, globalLogBeginner, textFileLogObserver, 
    FilteringLogObserver, LogLevelFilterPredicate)

from mqtt.client.factory import MQTTFactory

# ----------------
# Global variables
# ----------------

# Global object to control globally namespace logging
logLevelFilterPredicate = LogLevelFilterPredicate(defaultLogLevel=LogLevel.info)

BROKER = "tcp:test.mosquitto.org:1883"

# -----------------
# Utility Functions
# -----------------

def startLogging(console=True, filepath=None):
    '''
    Starts the global Twisted logger subsystem with maybe
    stdout and/or a file specified in the config file
    '''
    global logLevelFilterPredicate
   
    observers = []
    if console:
        observers.append( FilteringLogObserver(observer=textFileLogObserver(sys.stdout),  
            predicates=[logLevelFilterPredicate] ))
    
    if filepath is not None and filepath != "":
        observers.append( FilteringLogObserver(observer=textFileLogObserver(open(filepath,'a')), 
            predicates=[logLevelFilterPredicate] ))
    globalLogBeginner.beginLoggingTo(observers)


def setLogLevel(namespace=None, levelStr='info'):
    '''
    Set a new log level for a given namespace
    LevelStr is: 'critical', 'error', 'warn', 'info', 'debug'
    '''
    level = LogLevel.levelWithName(levelStr)
    logLevelFilterPredicate.setLogLevelForNamespace(namespace=namespace, level=level)

# -----------------------
# MQTT Subscriber Service
# ------------------------

class MQTTService(ClientService):


    def __init__(self, endpoint, factory):
        ClientService.__init__(self, endpoint, factory, retryPolicy=backoffPolicy())


    def startService(self):
        log.info("starting MQTT Client Subscriber Service")
        # invoke whenConnected() inherited method
        self.whenConnected().addCallback(self.connectToBroker)
        ClientService.startService(self)


    @inlineCallbacks
    def connectToBroker(self, protocol):
        '''
        Connect to MQTT broker
        '''
        self.protocol                 = protocol
        self.protocol.onPublish       = self.onPublish
        self.protocol.onDisconnection = self.onDisconnection
        self.protocol.setWindowSize(3)
        self.task = task.LoopingCall(self.publish)
        self.task.start(5.0, now=False) 
        try:
            yield self.protocol.connect("TwistedMQTT-pubsubs", keepalive=60)
            yield self.subscribe()
        except Exception as e:
            log.error("Connecting to {broker} raised {excp!s}", 
               broker=BROKER, excp=e)
        else:
            log.info("Connected and subscribed to {broker}", broker=BROKER)


    def publish(self):

        def _logFailure(failure):
            log.debug("publisher reported {message}", message=failure.getErrorMessage())
            return failure

        d1 = self.protocol.publish(topic="foo/bar/baz1", qos=1, message="hello world 1")
        d1.addErrback(_logFailure)       
        return d1


    def subscribe(self):

        def _logFailure(failure):
            log.debug("subscriber reported {message}", message=failure.getErrorMessage())
            return failure

        def _logGrantedQoS(value):
            log.debug("subscriber response {value!r}", value=value)
            return True

        d1 = self.protocol.subscribe("foo/bar/baz1", 2)
        d1.addCallbacks(_logGrantedQoS, _logFailure)      
        return d1


    def onPublish(self, topic, payload, qos, dup, retain, msgId):
        '''
        Callback Receiving messages from publisher
        '''
        log.debug("msg={payload}", payload=payload)


    def onDisconnection(self, reason):
        '''
        get notfied of disconnections
        and get a deferred for a new protocol object (next retry)
        '''
        log.debug("<Connection was lost !> <reason={r}>", r=reason)
        self.whenConnected().addCallback(self.connectToBroker)


if __name__ == '__main__':
    import sys
    log = Logger()
    startLogging()
    setLogLevel(namespace='mqtt',     levelStr='debug')
    setLogLevel(namespace='__main__', levelStr='debug')
    factory    = MQTTFactory(profile=MQTTFactory.PUBLISHER | MQTTFactory.SUBSCRIBER)
    myEndpoint = clientFromString(reactor, BROKER)
    serv       = MQTTService(myEndpoint, factory)
    serv.startService()
    reactor.run()
    


# ----------------------------------------------------------------------
# Copyright (C) 2015 by Rafael Gonzalez 
# #
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:

# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#----------------------------------------------------------------------

from twisted.trial    import unittest
from twisted.test     import proto_helpers
from twisted.internet import task, defer, error


from mqtt                   import v31
from mqtt.error             import MQTTWindowError
from mqtt.pdu               import CONNACK, PUBACK, PUBREC, PUBREL, PUBCOMP
from mqtt.client.base       import MQTTBaseProtocol
from mqtt.client.factory    import MQTTFactory
from mqtt.client.subscriber import MQTTProtocol as MQTTSubscriberProtocol
from mqtt.client.publisher  import MQTTProtocol as MQTTPublisherProtocol
from mqtt.client.pubsubs    import MQTTProtocol as MQTTPubSubsProtocol




class TestMQTTPublisherDisconnect(unittest.TestCase):
    '''
    Testing various cases of disconnect callback
    '''

    def setUp(self):
        '''
        Set up a connencted state
        '''
        self.transport = proto_helpers.StringTransportWithDisconnection()
        self.clock     = task.Clock()
        MQTTBaseProtocol.callLater = self.clock.callLater
        self.factory   = MQTTFactory(MQTTFactory.PUBLISHER | MQTTFactory.SUBSCRIBER)
        self._rebuild()
        self.disconnected = 0

    def _connect(self, cleanStart=True):
        '''
        Go to connected state
        '''
        ack = CONNACK()
        ack.session = False
        ack.resultCode = 0
        ack.encode()
        self.protocol.connect("TwistedMQTT-pubsubs", keepalive=0, cleanStart=cleanStart, version=v31)
        self.transport.clear()
        self.protocol.dataReceived(ack.encoded)

    def _disconnected(self, reason):
        self.disconnected += 1

    def _serverDown(self):
        self.transport.loseConnection()
        self.transport.clear()
        del self.protocol

    def _rebuild(self):
        self.protocol  = self.factory.buildProtocol(0)
        self.transport.protocol = self.protocol
        MQTTBaseProtocol.callLater = self.clock.callLater
        self.protocol.makeConnection(self.transport)

    def test_disconnect_1(self):
        '''Just connect and lose the transport'''
        self._connect()
        self.protocol.onDisconnection = self._disconnected
        self.transport.loseConnection()
        self.assertEqual(self.disconnected, 1)
        

    def test_disconnect_2(self):
        '''connect and disconnect'''
        self._connect()
        self.protocol.onDisconnection = self._disconnected
        self.protocol.disconnect()
        self.assertEqual(self.disconnected, 1)
        

    def test_disconnect_3(self):
        '''connect, generate a deferred and lose the transport'''
        self._connect()
        self.protocol.onDisconnection = self._disconnected
        d = self.protocol.publish(topic="foo/bar/baz1", qos=1, message="hello world 1")
        self.transport.clear()
        self.transport.loseConnection()
        self.assertEqual(self.disconnected, 1)
        self.failureResultOf(d).trap(error.ConnectionDone)
       

    def test_disconnect_4(self):
        '''connect, generate a deferred and disconnect'''
        self._connect()
        self.protocol.onDisconnection = self._disconnected
        d = self.protocol.publish(topic="foo/bar/baz1", qos=1, message="hello world 1")
        self.transport.clear()
        self.protocol.disconnect()
        self.assertEqual(self.disconnected, 1)
        self.failureResultOf(d).trap(error.ConnectionDone)
       

    def test_disconnect_5(self):
        '''connect with persistent session, generate a deferred and disconnect'''
        self._connect(cleanStart=False)
        self.protocol.onDisconnection = self._disconnected
        d = self.protocol.publish(topic="foo/bar/baz1", qos=1, message="hello world 1")
        self.transport.clear()
        self.protocol.disconnect()
        self.assertEqual(self.disconnected, 1)
        self.assertNoResult(d)
       

    def test_disconnect_6(self):
        '''connect with persistent session, generate a deferred , rebuilds protocol'''
        self._connect(cleanStart=False)
        self.protocol.onDisconnection = self._disconnected
        d = self.protocol.publish(topic="foo/bar/baz1", qos=1, message="hello world 1")
        self._serverDown()
        self._rebuild()
        self.assertEqual(self.disconnected, 1)
        self.assertNoResult(d)
       


class TestMQTTSubscriberDisconnect(unittest.TestCase):
    '''
    Testing various cases of disconnect callback
    '''

    def setUp(self):
        '''
        Set up a connencted state
        '''
        self.transport = proto_helpers.StringTransportWithDisconnection()
        self.clock     = task.Clock()
        MQTTBaseProtocol.callLater = self.clock.callLater
        self.factory   = MQTTFactory(MQTTFactory.PUBLISHER | MQTTFactory.SUBSCRIBER)
        self._rebuild()
        self.disconnected = 0

    def _connect(self, cleanStart=True):
        '''
        Go to connected state
        '''
        ack = CONNACK()
        ack.session = False
        ack.resultCode = 0
        ack.encode()
        self.protocol.connect("TwistedMQTT-sub", keepalive=0, cleanStart=cleanStart, version=v31)
        self.transport.clear()
        self.protocol.dataReceived(ack.encoded)

    def _disconnected(self, reason):
        self.disconnected += 1

    def _rebuild(self):
        self.protocol  = self.factory.buildProtocol(0)
        self.transport.protocol = self.protocol
        MQTTBaseProtocol.callLater = self.clock.callLater
        self.protocol.makeConnection(self.transport)

    def _serverDown(self):
        self.transport.loseConnection()
        self.transport.clear()
        del self.protocol

    def test_disconnect_1(self):
        '''Just connect and lose the transport'''
        self._connect()
        self.protocol.onDisconnection = self._disconnected
        self.transport.loseConnection()
        self.assertEqual(self.disconnected, 1)
       

    def test_disconnect_2(self):
        '''connect and disconnect'''
        self._connect()
        self.protocol.onDisconnection = self._disconnected
        self.protocol.disconnect()
        self.assertEqual(self.disconnected, 1)
       

    def test_disconnect_3(self):
        '''connect, generate a deferred and lose the transport'''
        self._connect()
        self.protocol.onDisconnection = self._disconnected
        d = self.protocol.subscribe("foo/bar/baz1", 2 )
        self.transport.clear()
        self.transport.loseConnection()
        self.assertEqual(self.disconnected, 1)
        self.failureResultOf(d).trap(error.ConnectionDone)

    def test_disconnect_4(self):
        '''connect, generate a deferred and disconnect'''
        self._connect()
        self.protocol.onDisconnection = self._disconnected
        d = self.protocol.subscribe("foo/bar/baz1", 2 )
        self.transport.clear()
        self.protocol.disconnect()
        self.assertEqual(self.disconnected, 1)
        self.failureResultOf(d).trap(error.ConnectionDone)

    def test_disconnect_5(self):
        '''connect with persistent session, 
        enerate a deferred that will not errback 
        and then disconnect'''
        self._connect(cleanStart=False)
        self.protocol.onDisconnection = self._disconnected
        d = self.protocol.subscribe("foo/bar/baz1", 2 )
        self.transport.clear()
        self.protocol.disconnect()
        self.assertEqual(self.disconnected, 1)
        self.assertNoResult(d)

    def test_disconnect_6(self):
        '''connect with persistent session, 
        generate a deferred that will not errback yet, 
        then rebuilds protocol'''
        self._connect(cleanStart=False)
        self.protocol.onDisconnection = self._disconnected
        d = self.protocol.subscribe("foo/bar/baz1", 2 )
        self._serverDown()
        self._rebuild()
        self.assertEqual(self.disconnected, 1)
        self.assertNoResult(d)





# ----------------------------------------------------------------------
# Copyright (C) 2015 by Rafael Gonzalez 
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
# 
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
# ----------------------------------------------------------------------

from twisted.trial    import unittest
from twisted.test     import proto_helpers
from twisted.internet import task, error
from twisted.python   import log


from mqtt import v31, v311
from mqtt.pdu import CONNACK, PINGREQ, PINGRES
from mqtt.client.factory    import MQTTFactory
from mqtt.client.subscriber import MQTTProtocol as MQTTSubscriberProtocol
from mqtt.client.publisher  import MQTTProtocol as MQTTPublisherProtocol
from mqtt.client.pubsubs    import MQTTProtocol as MQTTPubSubsProtocol

from mqtt.client.base       import MQTTBaseProtocol
from mqtt.error             import MQTTTimeoutError



class TestMQTTBaseProtocol1(unittest.TestCase):

    def setUp(self):
        self.transport = proto_helpers.StringTransportWithDisconnection()
        self.clock     = task.Clock()
        self.factory   = MQTTFactory(MQTTFactory.PUBLISHER)
        self.protocol  = MQTTBaseProtocol(self.factory)
        self.transport.protocol = self.protocol
        MQTTBaseProtocol.callLater = self.clock.callLater
        self.protocol.makeConnection(self.transport)
       

    def test_connect(self):
        self.CONNACK = CONNACK()
        self.CONNACK.session = False
        self.CONNACK.resultCode = 0
        self.CONNACK.encode()
        d = self.protocol.connect("TwistedMQTT-pub", keepalive=0, version=v31)
        self.transport.clear()
        self.assertEqual(self.protocol.state, self.protocol.CONNECTING)
        self.protocol.dataReceived(self.CONNACK.encoded)
        self.assertEqual(self.protocol.state, self.protocol.CONNECTED)
        self.assertEqual(self.CONNACK.session, self.successResultOf(d))
 

    def test_connect_timeout(self):
        d = self.protocol.connect("TwistedMQTT-pub", keepalive=0, version=v31)
        self.transport.clear()
        self.assertEqual(self.protocol.state, self.protocol.CONNECTING)
        self.assertNoResult(d)
        self.clock.advance(11)
        self.failureResultOf(d).trap(MQTTTimeoutError)

class TestMQTTBaseProtocol2(unittest.TestCase):

    def setUp(self):
        '''
        Set up a conencted state
        '''
        self.transport = proto_helpers.StringTransportWithDisconnection()
        self.clock     = task.Clock()
        MQTTBaseProtocol.callLater = self.clock.callLater
        self.factory   = MQTTFactory(MQTTFactory.SUBSCRIBER)
        self._rebuild()
      
    def tearDown(self):
        '''
        Needed because ping sets up a LoopingCall, outside Clock simulated callLater()
        '''
        self.transport.loseConnection()

    def _connect(self, keepalive=0, cleanStart=True):
        '''
        Go to connected state
        '''
        ack = CONNACK()
        ack.session = False
        ack.resultCode = 0
        ack.encode()
        self.protocol.connect("TwistedMQTT-sub", keepalive=keepalive, cleanStart=cleanStart, version=v31)
        self.transport.clear()
        self.protocol.dataReceived(ack.encoded)

    def _rebuild(self):
        self.protocol  = self.factory.buildProtocol(0)
        self.transport.protocol = self.protocol
        self.protocol.makeConnection(self.transport)


    def test_disconnect(self):
        self._connect()
        self.assertEqual(self.protocol.state, self.protocol.CONNECTED)
        self.protocol.disconnect()
        self.transport.clear()

    def test_ping(self):
        self._connect(keepalive=5)
        self.protocol.dataReceived(PINGRES().encode())
        self.transport.clear()
        self.assertEqual(self.protocol.state, self.protocol.CONNECTED)
      
    def test_ping_timeout(self):
        self._connect(keepalive=5)
        self.protocol.ping()
        self.transport.clear()
        self.clock.advance(6)
        self.assertEqual(self.protocol.state, self.protocol.IDLE)
        
class TestMQTTBaseExceptions(unittest.TestCase):

    def setUp(self):
        self.transport = proto_helpers.StringTransportWithDisconnection()
        self.clock     = task.Clock()
        self.factory   = MQTTFactory(MQTTFactory.PUBLISHER)
        self.protocol  = MQTTBaseProtocol(self.factory)
        self.transport.protocol = self.protocol
        MQTTBaseProtocol.callLater = self.clock.callLater
        self.protocol.makeConnection(self.transport)
       

    def test_connect_version(self):
        clientId="1234567890ABCDEFGHIJ1234567890"
        d = self.protocol.connect(clientId, keepalive=0, version=0)
        self.failureResultOf(d).trap(ValueError)
        self.assertEqual(self.protocol.state, self.protocol.IDLE)

    def test_connect_client_id_v31(self):
        clientId="1234567890ABCDEFGHIJ1234567890"
        d = self.protocol.connect(clientId, keepalive=0, version=v31)
        self.failureResultOf(d).trap(ValueError)
        self.assertEqual(self.protocol.state, self.protocol.IDLE)

    def test_connect_client_id_v311(self):
        clientId="1234567890ABCDEFGHIJ1234567890"*3000
        d = self.protocol.connect(clientId, keepalive=0, version=v311)
        self.failureResultOf(d).trap(ValueError)
        self.assertEqual(self.protocol.state, self.protocol.IDLE)

    def test_connect_keepalive_negative(self):
        clientId="1234567890A"
        d = self.protocol.connect(clientId, keepalive=-1, version=v31)
        self.failureResultOf(d).trap(ValueError)
        self.assertEqual(self.protocol.state, self.protocol.IDLE)

    def test_connect_keepalive_large(self):
        clientId="1234567890A"
        d = self.protocol.connect(clientId, keepalive=65537, version=v31)
        self.failureResultOf(d).trap(ValueError)
        self.assertEqual(self.protocol.state, self.protocol.IDLE)

    def test_connect_will_qos_large(self):
        clientId="1234567890A"
        d = self.protocol.connect(clientId, keepalive=0, version=v31, willQoS=5)
        self.failureResultOf(d).trap(ValueError)
        self.assertEqual(self.protocol.state, self.protocol.IDLE)

    def test_connect_will_qos_negative(self):
        clientId="1234567890A"
        d = self.protocol.connect(clientId, keepalive=0, version=v31, willQoS=-1)
        self.failureResultOf(d).trap(ValueError)
        self.assertEqual(self.protocol.state, self.protocol.IDLE)

    def test_connect_no_username_with_password(self):
        clientId="1234567890A"
        d = self.protocol.connect(clientId, keepalive=0, version=v31, password="foo")
        self.failureResultOf(d).trap(ValueError)
        self.assertEqual(self.protocol.state, self.protocol.IDLE)
    
    def test_connect_will_topic_no_message(self):
        clientId="1234567890A"
        d = self.protocol.connect(clientId, keepalive=0, version=v31, willTopic="foo/bar")
        self.failureResultOf(d).trap(ValueError)
        self.assertEqual(self.protocol.state, self.protocol.IDLE)

    def test_connect_will_message_no_topic(self):
        clientId="1234567890A"
        d = self.protocol.connect(clientId, keepalive=0, version=v31, willMessage="Hello")
        self.failureResultOf(d).trap(ValueError)
        self.assertEqual(self.protocol.state, self.protocol.IDLE)
    
    def test_window_size_large(self):
        self.assertRaises(ValueError, self.protocol.setWindowSize, 32)

    def test_window_size_negative(self):
        self.assertRaises(ValueError, self.protocol.setWindowSize, -1)

    def test_timeout_large(self):
        self.assertRaises(ValueError, self.protocol.setTimeout, 65536)

    def test_timeout_negative(self):
        self.assertRaises(ValueError, self.protocol.setTimeout, -1)
   
       
 
        


# ----------------------------------------------------------------------
# Copyright (C) 2015 by Rafael Gonzalez 
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
# 
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
# ----------------------------------------------------------------------

from twisted.trial import unittest
from twisted.test import proto_helpers

from mqtt.client.factory    import MQTTFactory
from mqtt.client.subscriber import MQTTProtocol as MQTTSubscriberProtocol
from mqtt.client.publisher  import MQTTProtocol as MQTTPublisherProtocol
from mqtt.client.pubsubs    import MQTTProtocol as MQTTPubSubsProtocol


class PDUTestCase(unittest.TestCase):

    
    def test_buildProtocol_publisher(self):
        self.factory = MQTTFactory(MQTTFactory.PUBLISHER)
        p = self.factory.buildProtocol(0)
        self.assertIsInstance(p, MQTTPublisherProtocol)
        
    
    def test_buildProtocol_subscriber(self):
        self.factory = MQTTFactory(MQTTFactory.SUBSCRIBER)
        p = self.factory.buildProtocol(0)
        self.assertIsInstance(p, MQTTSubscriberProtocol)
        

    def test_buildProtocol_pubsubs(self):
        self.factory = MQTTFactory(MQTTFactory.SUBSCRIBER + 
            MQTTFactory.PUBLISHER)
        p = self.factory.buildProtocol(0)
        self.assertIsInstance(p, MQTTPubSubsProtocol)

    def test_buildProtocol_other(self):
        self.factory = MQTTFactory(0)
        self.assertRaises(ValueError, self.factory.buildProtocol, 0)



# ----------------------------------------------------------------------
# Copyright (C) 2015 by Rafael Gonzalez 
# #
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:

# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#----------------------------------------------------------------------

from twisted.trial    import unittest
from twisted.test     import proto_helpers
from twisted.internet import task, defer, error
from twisted.python   import log


from mqtt                   import v31
from mqtt.error             import MQTTWindowError
from mqtt.pdu               import CONNACK, PUBACK, PUBREC, PUBREL, PUBCOMP
from mqtt.client.base       import MQTTBaseProtocol, MQTTStateError
from mqtt.client.factory    import MQTTFactory
from mqtt.client.subscriber import MQTTProtocol as MQTTSubscriberProtocol
from mqtt.client.publisher  import MQTTProtocol as MQTTPublisherProtocol
from mqtt.client.pubsubs    import MQTTProtocol as MQTTPubSubsProtocol

from twisted.internet.address import IPv4Address



class TestMQTTPublisher1(unittest.TestCase):


    def setUp(self):
        '''
        Set up a conencted state
        '''
        self.transport = proto_helpers.StringTransportWithDisconnection()
        self.clock     = task.Clock()
        MQTTBaseProtocol.callLater = self.clock.callLater
        self.factory   = MQTTFactory(MQTTFactory.PUBLISHER)
        self._rebuild()
        # Just to generate connection contexts
        

    def _connect(self, cleanStart=True):
        '''
        Go to connected state
        '''
        ack = CONNACK()
        ack.session = False
        ack.resultCode = 0
        ack.encode()
        self.protocol.connect("TwistedMQTT-pub", keepalive=0, cleanStart=cleanStart, version=v31)
        self.transport.clear()
        self.protocol.dataReceived(ack.encoded)


    def _serverDown(self):
        self.transport.loseConnection()
        self.transport.clear()
        del self.protocol

    def _rebuild(self):
        self.addr = IPv4Address('TCP','localhost',1880)
        self.protocol  = self.factory.buildProtocol(self.addr)
        self.transport.protocol = self.protocol
        MQTTBaseProtocol.callLater = self.clock.callLater
        self.protocol.makeConnection(self.transport)


    def _publish(self, n, qos, topic, msg, window=None):
        if window is None:
            self.protocol.setWindowSize(n)
        else:
            self.protocol.setWindowSize(window)
        dl = []
        for i in range(0,n):
            dl.append(self.protocol.publish(topic=topic, qos=qos, message=msg))
        self.transport.clear()
        for d in dl:
            if qos == 0:
                self.assertEqual(None, self.successResultOf(d))
            else:
                self.assertNoResult(d)
        return dl
    
    def _puback(self, dl):
        ackl = []
        for i in range(0, len(dl)):
            ack= PUBACK()
            ack.msgId = dl[i].msgId
            ackl.append(ack)
        encoded = bytearray()
        for ack in ackl:
            encoded.extend(ack.encode())
        self.protocol.dataReceived(encoded)
        self.transport.clear()
        for i in range(0, len(dl)):
            self.assertEqual(dl[i].msgId, self.successResultOf(dl[i]))

    def _pubrec(self, dl):
        recl = []
        for i in range(0, len(dl)):
            rec= PUBREC()
            rec.msgId = dl[i].msgId
            recl.append(rec)
        encoded = bytearray()
        for rec in recl:
            encoded.extend(rec.encode())
        self.protocol.dataReceived(encoded)
        self.transport.clear()
        for i in range(0, len(dl)):
            self.assertNoResult(dl[i])


    def _pubcomp(self, dl):
        compl = []
        for i in range(0, len(dl)):
            comp= PUBCOMP()
            comp.msgId = dl[i].msgId
            compl.append(comp)
        encoded = bytearray()
        for comp in compl:
            encoded.extend(comp.encode())
        self.protocol.dataReceived(encoded)
        self.transport.clear()
        for i in range(0, len(dl)):
            self.assertEqual(dl[i].msgId, self.successResultOf(dl[i]))



    def test_publish_single_qos0(self):
        self._connect()
        d = self.protocol.publish(topic="foo/bar/baz1", qos=0, message="hello world 0")
        self.assertEqual(len(self.protocol.factory.windowPublish[self.addr]),  0)
        self.assertEqual(None, self.successResultOf(d))

    def test_publish_single_qos1(self):
        self._connect()
        d = self.protocol.publish(topic="foo/bar/baz1", qos=1, message="hello world 1")
        self.assertEqual(len(self.protocol.factory.windowPublish[self.addr]),  1)
        self.transport.clear()
        ack = PUBACK()
        ack.msgId = d.msgId
        self.protocol.dataReceived(ack.encode())
        self.assertEqual(len(self.protocol.factory.windowPublish[self.addr]),  0)
        self.assertEqual(ack.msgId, self.successResultOf(d))

    def test_publish_single_qos2(self):
        self._connect()
        d = self.protocol.publish(topic="foo/bar/baz1", qos=2, message="hello world 2")
        self.assertEqual(len(self.protocol.factory.windowPublish[self.addr]),  1)
        self.assertEqual(len(self.protocol.factory.windowPubRelease[self.addr]), 0)
        self.transport.clear()
        ack = PUBREC()
        ack.msgId = d.msgId
        self.protocol.dataReceived(ack.encode())
        self.transport.clear()
        self.assertEqual(len(self.protocol.factory.windowPublish[self.addr]),  0)
        self.assertEqual(len(self.protocol.factory.windowPubRelease[self.addr]), 1)
        ack = PUBCOMP()
        ack.msgId = d.msgId
        self.protocol.dataReceived(ack.encode())
        self.assertEqual(len(self.protocol.factory.windowPublish[self.addr]),  0)
        self.assertEqual(len(self.protocol.factory.windowPubRelease[self.addr]), 0)
        self.assertEqual(ack.msgId, self.successResultOf(d))

    def test_publish_several_qos0(self):
        self._connect()
        dl = self._publish(n=3, qos=0, topic="foo/bar/baz", msg="Hello World")
        self.assertEqual(len(self.protocol.factory.windowPublish[self.addr]),  0)
        

    def test_publish_several_qos1(self):
        self._connect()
        dl = self._publish(n=3, qos=1, topic="foo/bar/baz", msg="Hello World")
        self.assertEqual(len(self.protocol.factory.windowPublish[self.addr]),  len(dl))
        self._puback(dl)
        self.assertEqual(len(self.protocol.factory.windowPublish[self.addr]),  0)
        

    def test_publish_several_qos2(self):
        self._connect()
        dl = self._publish(n=3, qos=2, topic="foo/bar/baz", msg="Hello World")
        self.assertEqual(len(self.protocol.factory.windowPublish[self.addr]),  len(dl))
        self.assertEqual(len(self.protocol.factory.windowPubRelease[self.addr]), 0)
        self._pubrec(dl)
        self.assertEqual(len(self.protocol.factory.windowPublish[self.addr]),  0)
        self.assertEqual(len(self.protocol.factory.windowPubRelease[self.addr]), len(dl))
        self._pubcomp(dl)
        self.assertEqual(len(self.protocol.factory.windowPublish[self.addr]),  0)
        self.assertEqual(len(self.protocol.factory.windowPubRelease[self.addr]), 0)


    def test_publish_many_qos1(self):
        '''
        Test enqueuing when not all ACKs arrives
        '''
        self._connect()
        window = 3
        n = 7
        dl = self._publish(n=n, window=window, qos=1, topic="foo/bar/baz", msg="Hello World")
        self.assertEqual(len(self.protocol.factory.windowPublish[self.addr]),  window)
        self.assertEqual(len(self.protocol.factory.queuePublishTx[self.addr]), n-window)
        self._puback(dl[0:window])
        self.assertEqual(len(self.protocol.factory.windowPublish[self.addr]),  window)
        self.assertEqual(len(self.protocol.factory.queuePublishTx[self.addr]), 1)
        self._puback(dl[window:2*window])
        self.assertEqual(len(self.protocol.factory.windowPublish[self.addr]),  1)
        self.assertEqual(len(self.protocol.factory.queuePublishTx[self.addr]), 0)
        self._puback(dl[2*window:])
        self.assertEqual(len(self.protocol.factory.windowPublish[self.addr]),  0)
        self.assertEqual(len(self.protocol.factory.queuePublishTx[self.addr]), 0)

    def test_publish_many_qos2(self):
        '''
        Test enqueuing when not all ACKs arrives
        '''
        self._connect()
        window = 3
        n = 7
        dl = self._publish(n=n, window=window, qos=2, topic="foo/bar/baz", msg="Hello World")
        self.assertEqual(len(self.protocol.factory.windowPublish[self.addr]),  window)
        self.assertEqual(len(self.protocol.factory.queuePublishTx[self.addr]), n-window)
        self._pubrec(dl[0:window])
        self.assertEqual(len(self.protocol.factory.windowPublish[self.addr]),  0)
        self.assertEqual(len(self.protocol.factory.windowPubRelease[self.addr]), window)
        self._pubcomp(dl[0:window])
        self.assertEqual(len(self.protocol.factory.queuePublishTx[self.addr]), n-2*window)
        self.assertEqual(len(self.protocol.factory.windowPublish[self.addr]),  window)
        self.assertEqual(len(self.protocol.factory.windowPubRelease[self.addr]), 0)



    def test_lost_session(self):
        self._connect()
        dl = self._publish(n=3, qos=2, topic="foo/bar/baz", msg="Hello World")
        self.assertEqual(len(self.protocol.factory.windowPublish[self.addr]),  len(dl))
        self.assertEqual(len(self.protocol.factory.windowPubRelease[self.addr]), 0)
        self._serverDown()
        self.assertEqual(len(self.factory.windowPublish[self.addr]),  0)
        self.assertEqual(len(self.factory.windowPubRelease[self.addr]), 0)
        for d in dl:
            self.failureResultOf(d).trap(error.ConnectionDone)
       

    def test_persistent_session_qos1(self):
        self._connect(cleanStart=False)
        dl = self._publish(n=3, qos=1, topic="foo/bar/baz", msg="Hello World")
        self.assertEqual(len(self.protocol.factory.windowPublish[self.addr]),  len(dl))
        self._serverDown()
        self.assertEqual(len(self.factory.windowPublish[self.addr]),  len(dl))
        for d in dl:
            self.assertNoResult(d)
        self._rebuild()
        self._connect(cleanStart=False)
        self.assertEqual(len(self.protocol.factory.windowPublish[self.addr]),  len(dl))
        self._puback(dl)
        self.assertEqual(len(self.protocol.factory.windowPublish[self.addr]),  0)


    def test_persistent_session_qos2(self):
        self._connect(cleanStart=False)
        dl = self._publish(n=3, qos=2, topic="foo/bar/baz", msg="Hello World")
        self.assertEqual(len(self.protocol.factory.windowPublish[self.addr]),  len(dl))
        self.assertEqual(len(self.protocol.factory.windowPubRelease[self.addr]), 0)
        self._serverDown()
        for d in dl:
            self.assertNoResult(d)
        self._rebuild()
        self._connect(cleanStart=False)
        self.assertEqual(len(self.factory.windowPublish[self.addr]),  len(dl))
        self.assertEqual(len(self.protocol.factory.windowPubRelease[self.addr]), 0)
        self._pubrec(dl)
        self.assertEqual(len(self.factory.windowPublish[self.addr]), 0 )
        self.assertEqual(len(self.protocol.factory.windowPubRelease[self.addr]), len(dl))
        self._pubcomp(dl)
        self.assertEqual(len(self.factory.windowPublish[self.addr]),  0)
        self.assertEqual(len(self.protocol.factory.windowPubRelease[self.addr]), 0)



    def test_persistent_release_qos2(self):
        self._connect(cleanStart=False)
        dl = self._publish(n=3, qos=2, topic="foo/bar/baz", msg="Hello World")
        #- generate two ACK and simulate a server disconnect with a new client protocol
        # being built on the client 
        self._pubrec(dl[:-1])   # Only the first two

        self._serverDown()
        self.assertNoResult(dl[0])
        self.assertNoResult(dl[1])
        self.assertNoResult(dl[2])
        self._rebuild()
        self.assertEqual(len(self.protocol.factory.windowPublish[self.addr]),  1)
        self.assertEqual(len(self.protocol.factory.windowPubRelease[self.addr]),  2)
        # Reconnect with the new client protcol object
        self._connect(cleanStart=False)
        self._pubrec(dl[-1:])   # send the last one
        self.assertEqual(len(self.protocol.factory.windowPublish[self.addr]), 0)
        self.assertEqual(len(self.protocol.factory.windowPubRelease[self.addr]), 3)
        self._pubcomp(dl[0:1])   # send the first comp
        self.assertEqual(len(self.factory.windowPublish[self.addr]),  0)
        self.assertEqual(len(self.protocol.factory.windowPubRelease[self.addr]), 2)
        self._pubcomp(dl[1:2])   # send the second comp
        self.assertEqual(len(self.factory.windowPublish[self.addr]),  0)
        self.assertEqual(len(self.protocol.factory.windowPubRelease[self.addr]), 1)
        self._pubcomp(dl[-1:])   # send the last comp
        self.assertEqual(len(self.factory.windowPublish[self.addr]),  0)
        self.assertEqual(len(self.protocol.factory.windowPubRelease[self.addr]), 0)


class TestMQTTPublisherDisconnect(unittest.TestCase):
    '''
    Testing various cases of disconnect callback
    '''

    def setUp(self):
        '''
        Set up a connencted state
        '''
        self.transport = proto_helpers.StringTransportWithDisconnection()
        self.clock     = task.Clock()
        MQTTBaseProtocol.callLater = self.clock.callLater
        self.factory   = MQTTFactory(MQTTFactory.PUBLISHER)
        self._rebuild()
        self.disconnected = False

    def _connect(self, cleanStart=True):
        '''
        Go to connected state
        '''
        ack = CONNACK()
        ack.session = False
        ack.resultCode = 0
        ack.encode()
        self.protocol.connect("TwistedMQTT-pub", keepalive=0, cleanStart=cleanStart, version=v31)
        self.transport.clear()
        self.protocol.dataReceived(ack.encoded)

    def _disconnected(self, reason):
        self.disconnected = True

    def _serverDown(self):
        self.transport.loseConnection()
        self.transport.clear()
        del self.protocol

    def _rebuild(self):
        self.protocol  = self.factory.buildProtocol(0)
        self.transport.protocol = self.protocol
        MQTTBaseProtocol.callLater = self.clock.callLater
        self.protocol.makeConnection(self.transport)

    def test_disconnect_1(self):
        '''Just connect and lose the transport'''
        self._connect()
        self.protocol.onDisconnection = self._disconnected
        self.transport.loseConnection()
        self.assertEqual(self.disconnected, True)

    def test_disconnect_2(self):
        '''connect and disconnect'''
        self._connect()
        self.protocol.onDisconnection = self._disconnected
        self.protocol.disconnect()
        self.assertEqual(self.disconnected, True)

    def test_disconnect_3(self):
        '''connect, generate a deferred and lose the transport'''
        self._connect()
        self.protocol.onDisconnection = self._disconnected
        d = self.protocol.publish(topic="foo/bar/baz1", qos=1, message="hello world 1")
        self.transport.clear()
        self.transport.loseConnection()
        self.assertEqual(self.disconnected, True)
        self.failureResultOf(d).trap(error.ConnectionDone)

    def test_disconnect_4(self):
        '''connect, generate a deferred and disconnect'''
        self._connect()
        self.protocol.onDisconnection = self._disconnected
        d = self.protocol.publish(topic="foo/bar/baz1", qos=1, message="hello world 1")
        self.transport.clear()
        self.protocol.disconnect()
        self.assertEqual(self.disconnected, True)
        self.failureResultOf(d).trap(error.ConnectionDone)

    def test_disconnect_5(self):
        '''connect with persistent session, generate a deferred and disconnect'''
        self._connect(cleanStart=False)
        self.protocol.onDisconnection = self._disconnected
        d = self.protocol.publish(topic="foo/bar/baz1", qos=1, message="hello world 1")
        self.transport.clear()
        self.protocol.disconnect()
        self.assertEqual(self.disconnected, True)
        self.assertNoResult(d)

    def test_disconnect_6(self):
        '''connect with persistent session, generate a deferred , rebuilds protocol'''
        self._connect(cleanStart=False)
        self.protocol.onDisconnection = self._disconnected
        d = self.protocol.publish(topic="foo/bar/baz1", qos=1, message="hello world 1")
        self._serverDown()
        self._rebuild()
        self.assertEqual(self.disconnected, True)
        self.assertNoResult(d)


class TestMQTTPublisherForbiddenOps(unittest.TestCase):
    '''
    Testing various cases of disconnect callback
    '''

    def setUp(self):
        '''
        Set up a connencted state
        '''
        self.transport = proto_helpers.StringTransportWithDisconnection()
        self.clock     = task.Clock()
        MQTTBaseProtocol.callLater = self.clock.callLater
        self.factory   = MQTTFactory(MQTTFactory.PUBLISHER)
        self._rebuild()
        self.disconnected = False
        self._rebuild()
        self._connect()

    def _connect(self, cleanStart=True):
        '''
        Go to connected state
        '''
        ack = CONNACK()
        ack.session = False
        ack.resultCode = 0
        ack.encode()
        self.protocol.connect("TwistedMQTT-pub", keepalive=0, cleanStart=cleanStart, version=v31)
        self.transport.clear()
        self.protocol.dataReceived(ack.encoded)


    def _rebuild(self):
        self.protocol  = self.factory.buildProtocol(0)
        self.transport.protocol = self.protocol
        MQTTBaseProtocol.callLater = self.clock.callLater
        self.protocol.makeConnection(self.transport)

    def test_forbidden_subscribe(self):
        '''Just connect and lose the transport'''
        d = self.protocol.subscribe("foo/bar/baz1", 2 )
        self.failureResultOf(d).trap(MQTTStateError)
      
    def test_forbidden_unsubscribe(self):
        '''Just connect and lose the transport'''
        d = self.protocol.unsubscribe("foo/bar/baz1")
        self.failureResultOf(d).trap(MQTTStateError)

    def test_forbidden_publish_callback(self):
        '''Just connect and lose the transport'''
        def onPublish(topic, payload, qos, dup, retain, msgId):
            pass
        self.assertRaises(MQTTStateError, self.protocol.onPublish, onPublish)



class TestMQTTPublisherIntervals(unittest.TestCase):


    def setUp(self):
        '''
        Set up a conencted state
        '''
        self.transport = proto_helpers.StringTransportWithDisconnection()
        self.clock     = task.Clock()
        MQTTBaseProtocol.callLater = self.clock.callLater
        self.factory   = MQTTFactory(MQTTFactory.PUBLISHER)
        self.addr = IPv4Address('TCP','localhost',1880)
        self._rebuild()
        # Just to generate connection contexts
        

    def _connect(self, cleanStart=True):
        '''
        Go to connected state
        '''
        ack = CONNACK()
        ack.session = False
        ack.resultCode = 0
        ack.encode()
        self.protocol.connect("TwistedMQTT-pub", keepalive=0, cleanStart=cleanStart, version=v31)
        self.transport.clear()
        self.protocol.dataReceived(ack.encoded)

    def _rebuild(self):
        self.protocol  = self.factory.buildProtocol(self.addr)
        self.transport.protocol = self.protocol
        MQTTBaseProtocol.callLater = self.clock.callLater
        self.protocol.makeConnection(self.transport)


    def _publish(self, n, qos, topic, msg, window=None):
        if window is None:
            self.protocol.setWindowSize(n)
        else:
            self.protocol.setWindowSize(window)
        dl = []
        for i in range(0,n):
            dl.append(self.protocol.publish(topic=topic, qos=qos, message=msg))
        self.transport.clear()
        for d in dl:
            if qos == 0:
                self.assertEqual(None, self.successResultOf(d))
            else:
                self.assertNoResult(d)
        return dl
    
    def _puback(self, dl):
        ackl = []
        for i in range(0, len(dl)):
            ack= PUBACK()
            ack.msgId = dl[i].msgId
            ackl.append(ack)
        encoded = bytearray()
        for ack in ackl:
            encoded.extend(ack.encode())
        self.protocol.dataReceived(encoded)
        self.transport.clear()
        for i in range(0, len(dl)):
            self.assertEqual(dl[i].msgId, self.successResultOf(dl[i]))

    def test_publish_very_large_qos1(self):
        message = '0123456789ABCDEF'*1000000 # Large PDU
        self._connect()

        # Test at 1MByte/sec
        self.protocol.setBandwith(1000000.0)
        d = self.protocol.publish(topic="foo/bar/baz1", qos=1, message=message)
        self.assertEqual(self.protocol.factory.windowPublish[self.addr][1].dup, False)
        self.transport.clear()
        self.clock.advance(10)
        self.assertEqual(self.protocol.factory.windowPublish[self.addr][1].dup, False)
        ack = PUBACK()
        ack.msgId = d.msgId
        self.protocol.dataReceived(ack.encode())
        self.transport.clear()
        self.assertEqual(ack.msgId, self.successResultOf(d))

        # A large PDU with a large bandwith estimation may retransmit
        self.protocol.setBandwith(10000000.0)
        d = self.protocol.publish(topic="foo/bar/baz1", qos=1, message=message)
        self.assertEqual(self.protocol.factory.windowPublish[self.addr][2].dup, False)
        self.transport.clear()
        self.clock.advance(10)
        self.assertEqual(self.protocol.factory.windowPublish[self.addr][2].dup, True)
        ack = PUBACK()
        ack.msgId = d.msgId
        self.protocol.dataReceived(ack.encode())
        self.transport.clear()
        self.assertEqual(ack.msgId, self.successResultOf(d))

    




import sys

from twisted.internet             import reactor, task
from twisted.internet.defer       import inlineCallbacks, DeferredList
from twisted.application.internet import ClientService, backoffPolicy
from twisted.internet.endpoints   import clientFromString
from twisted.logger   import (
    Logger, LogLevel, globalLogBeginner, textFileLogObserver, 
    FilteringLogObserver, LogLevelFilterPredicate)

from mqtt.client.factory import MQTTFactory

# ----------------
# Global variables
# ----------------

# Global object to control globally namespace logging
logLevelFilterPredicate = LogLevelFilterPredicate(defaultLogLevel=LogLevel.info)

BROKER = "tcp:test.mosquitto.org:1883"

# -----------------
# Utility Functions
# -----------------

def startLogging(console=True, filepath=None):
    '''
    Starts the global Twisted logger subsystem with maybe
    stdout and/or a file specified in the config file
    '''
    global logLevelFilterPredicate
   
    observers = []
    if console:
        observers.append( FilteringLogObserver(observer=textFileLogObserver(sys.stdout),  
            predicates=[logLevelFilterPredicate] ))
    
    if filepath is not None and filepath != "":
        observers.append( FilteringLogObserver(observer=textFileLogObserver(open(filepath,'a')), 
            predicates=[logLevelFilterPredicate] ))
    globalLogBeginner.beginLoggingTo(observers)


def setLogLevel(namespace=None, levelStr='info'):
    '''
    Set a new log level for a given namespace
    LevelStr is: 'critical', 'error', 'warn', 'info', 'debug'
    '''
    level = LogLevel.levelWithName(levelStr)
    logLevelFilterPredicate.setLogLevelForNamespace(namespace=namespace, level=level)


# -----------------------
# MQTT Publishing Service
# -----------------------

class MQTTService(ClientService):

    def __init__(self, endpoint, factory):
        ClientService.__init__(self, endpoint, factory, retryPolicy=backoffPolicy())


    def startService(self):
        log.info("starting MQTT Client Publisher Service")
        # invoke whenConnected() inherited method
        self.whenConnected().addCallback(self.connectToBroker)
        ClientService.startService(self)


    @inlineCallbacks
    def connectToBroker(self, protocol):
        '''
        Connect to MQTT broker
        '''
        self.protocol                 = protocol
        self.protocol.onDisconnection = self.onDisconnection
        # We are issuing 3 publish in a row
        # if order matters, then set window size to 1
        # Publish requests beyond window size are enqueued
        self.protocol.setWindowSize(3) 
        self.task = task.LoopingCall(self.publish)
        self.task.start(5.0, now=False)
        try:
            yield self.protocol.connect("TwistedMQTT-pub", keepalive=60)
        except Exception as e:
            log.error("Connecting to {broker} raised {excp!s}", 
               broker=BROKER, excp=e)
        else:
            log.info("Connected to {broker}", broker=BROKER)


    def onDisconnection(self, reason):
        '''
        get notfied of disconnections
        and get a deferred for a new protocol object (next retry)
        '''
        log.debug("<Connection was lost !> <reason={r}>", r=reason)
        self.whenConnected().addCallback(self.connectToBroker)


    def publish(self):


        def _logFailure(failure):
            log.debug("reported {message}", message=failure.getErrorMessage())
            return failure

        def _logAll(*args):
            log.debug("all publihing complete args={args!r}",args=args)

        log.debug(" >< Starting one round of publishing >< ")
        d1 = self.protocol.publish(topic="foo/bar/baz1", qos=0, message="hello world 0")
        d1.addErrback(_logFailure)
        d2 = self.protocol.publish(topic="foo/bar/baz2", qos=1, message="hello world 1")
        d2.addErrback(_logFailure)
        d3 = self.protocol.publish(topic="foo/bar/baz3", qos=2, message="hello world 2")
        d3.addErrback(_logFailure)
        dlist = DeferredList([d1,d2,d3], consumeErrors=True)
        dlist.addCallback(_logAll)
        return dlist



if __name__ == '__main__':
    import sys
    log = Logger()
    startLogging()
    setLogLevel(namespace='mqtt',     levelStr='debug')
    setLogLevel(namespace='__main__', levelStr='debug')

    factory    = MQTTFactory(profile=MQTTFactory.PUBLISHER)
    myEndpoint = clientFromString(reactor, BROKER)
    serv       = MQTTService(myEndpoint, factory)
    serv.startService()
    reactor.run()
